/*
FILE:     CifBuildReorganizer.C
*/
/*
VERSION:  2.000
*/
/*
DATE:     7/13/2006
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2006 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "CifBuildReorganizer.h"

extern FileNavigator * fnav;
extern Word binSearch(const string& key, const vector<string>& base,
         size_t size, int (*compar)(const string&, const string&));

int sCompare(const string& a, const string& b) {
/*
-------------------------------------------------------------------------------
   sCompare(): (string compare) is a function used by binSearch() to search an
   array of strings.
-------------------------------------------------------------------------------
*/
  return strcmp((a.c_str()), (b.c_str())); 
}

CategoryBuildObj::CategoryBuildObj() {
/*
-------------------------------------------------------------------------------
  constructor that initializes the object
-------------------------------------------------------------------------------
*/
  Clear();
}

CategoryBuildObj::CategoryBuildObj(const string& dataBlockName, const string& categoryId) {
/*
-------------------------------------------------------------------------------
  constructor that initializes the object and sets the keys
-------------------------------------------------------------------------------
*/
  Clear();
  SetId(dataBlockName, categoryId);
}  

void CategoryBuildObj::SetId(const string& dataBlockName, const string& categoryId)
{/*
-------------------------------------------------------------------------------
  SetId() sets the keys of the object
-------------------------------------------------------------------------------
*/

  _id = categoryId;  
  _datablock = dataBlockName;

}

int CategoryBuildObj::GetObject(uWord which) {
/*
-------------------------------------------------------------------------------
  GetObject() calls all of the private Get methods so that the entire object
  can be retrieved from memory. The object# must be specified.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index) free(_index);
  uWord num;
  _index = fnav->GetWords(which, num, err); 
  GetDescription();
  GetMandatoryCode();
  GetKeyItems();
  GetExamplesCase();
  GetExamplesDetail();
  GetCategoryGroups();
  GetCategoryMethods();
  GetDescriptionNDB();
  GetExamplesCaseNDB();
  GetExamplesDetailNDB();

  if (err == NO_ERROR) 
    return NO_ERROR; 
  else return err;
}

/*
  The following Get methods will retrieve the data members from persistent
  storage by calling on the FileNavigator class.
*/

int CategoryBuildObj::GetDescription() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_description.empty()) _description.clear();
  if (_index[0] == 0) _description.clear();
  else fnav->GetString(_description, _index[0], err);
  return err; 
}

int CategoryBuildObj::GetDescriptionNDB() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_descriptionNDB.empty()) _descriptionNDB.clear();
  if (_index[7] == 0) _descriptionNDB.clear();
  else fnav->GetString(_descriptionNDB, _index[7], err);
  return err; 
}

int CategoryBuildObj::GetMandatoryCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_mandatoryCode.empty()) _mandatoryCode.empty();
  if (_index[1] == 0) _mandatoryCode.empty();
  else fnav->GetString(_mandatoryCode, _index[1], err);
  return err; 
}

int CategoryBuildObj::GetKeyItems() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_keyItems.empty()) {
    for (uWord i = 0; i < _keyItems.size(); i++) 
      _keyItems[i].clear();
    _keyItems.clear();
  }
  if (_index[2] == 0) _keyItems.clear();
  else fnav->GetStrings(_keyItems, _index[2], err);
  return err; 
}

int CategoryBuildObj::GetExamplesCase() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesCase.empty()) {
    for (uWord i = 0; i < _examplesCase.size(); i++) 
      _examplesCase[i].clear();
    _examplesCase.clear();
  }
  if (_index[3] == 0) _examplesCase.clear();
  else fnav->GetStrings(_examplesCase, _index[3], err);
  return err; 
}

int CategoryBuildObj::GetExamplesDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesDetail.empty()) {
    for (uWord i = 0; i < _examplesDetail.size(); i++) 
      _examplesDetail[i].clear();
    _examplesDetail.clear();
  }
  if (_index[4] == 0) _examplesDetail.clear();
  else fnav->GetStrings(_examplesDetail, _index[4], err);
  return err; 
}

int CategoryBuildObj::GetExamplesCaseNDB() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesCaseNDB.empty()) {
    for (uWord i = 0; i < _examplesCaseNDB.size(); i++) 
      _examplesCaseNDB[i].clear();
    _examplesCaseNDB.clear();
  }
  if (_index[8] == 0) _examplesCaseNDB.clear();
  else fnav->GetStrings(_examplesCaseNDB, _index[8], err);
  return err; 
}

int CategoryBuildObj::GetExamplesDetailNDB() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesDetailNDB.empty()) {
    for (uWord i = 0; i < _examplesDetailNDB.size(); i++) 
      _examplesDetailNDB[i].clear();
    _examplesDetailNDB.clear();
  }
  if (_index[9] == 0) _examplesDetailNDB.clear();
  else fnav->GetStrings(_examplesDetailNDB, _index[9], err);
  return err; 
}

int CategoryBuildObj::GetCategoryGroups() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categoryGroups.empty()) {
    for (uWord i = 0; i < _categoryGroups.size(); i++) 
      _categoryGroups[i].clear();
    _categoryGroups.clear();
  }
  if (_index[5] == 0) _categoryGroups.clear();
  else fnav->GetStrings(_categoryGroups, _index[5], err);
  return err; 
}

int CategoryBuildObj::GetCategoryMethods() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categoryMethods.empty()) {
    for (uWord i = 0; i < _categoryMethods.size(); i++) 
      _categoryMethods[i].clear();
    _categoryMethods.clear();
  }
  if (_index[6] == 0) _categoryMethods.clear();
  else fnav->GetStrings(_categoryMethods, _index[6], err);
  return err; 
}


  
ItemBuildObj::ItemBuildObj() {
/*
-------------------------------------------------------------------------------
  constructor that initializes the object
-------------------------------------------------------------------------------
*/
  Clear();
}

ItemBuildObj::ItemBuildObj(const string& dataBlockName, const string& itemName) {
/*
-------------------------------------------------------------------------------
  constructor that initializes the object and sets the keys
-------------------------------------------------------------------------------
*/
  Clear();
  SetName(dataBlockName, itemName);
}

void ItemBuildObj::SetName(const string& dataBlockName, const string& itemName) {
/*
-------------------------------------------------------------------------------
  SetName() will set the keys of the object
-------------------------------------------------------------------------------
*/
  _name = itemName;
  _datablock = dataBlockName;
}

int ItemBuildObj::GetObject(uWord which) {
/*
-------------------------------------------------------------------------------
  GetObject() will call all of the private Get methods so that the entire
  object can be retrieved from persistent storage.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index) free(_index);
  uWord num;
  _index = fnav->GetWords(which, num, err); 
  GetDescription();
  GetType();
  GetPrimitiveCode();
  GetRegex();
  GetExamplesCase();
  GetExamplesDetail();
  GetEnumeration();
  GetEnumerationDetail();
  GetMandatoryCode();
  GetDefaultValue();
  GetUnits();
  GetRangeMin();
  GetRangeMax();
  GetAliases();
  GetAliasesDictionary();
  GetAliasesDictionaryVersion();
  GetItemStructure();
  GetItemStructureOrganization();
  GetDependents();
  GetRelated();
  GetRelatedFunctionCode();
  GetSubcategories();
  GetLinkedChildren();
  GetLinkedParents();
  GetMethods();
  GetTypeConditions();
  GetDescriptionNDB();
  GetExamplesCaseNDB();
  GetExamplesDetailNDB();
  GetEnumerationNDB();
  GetEnumerationDetailNDB();

  if (err == NO_ERROR) 
    return NO_ERROR; 
  else return err;
}

/*
  The following Get methods will retrieve the data members from persistent
  storage by calling on the FileNavigator class.
*/

int ItemBuildObj::GetDescription() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_description.empty()) _description.clear();
  if (_index[0] == 0) _description.clear();
  else fnav->GetString(_description, _index[0], err);
  return err; 
}

int ItemBuildObj::GetDescriptionNDB() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_descriptionNDB.empty()) _descriptionNDB.clear();
  if (_index[26] == 0) _descriptionNDB.clear();
  else fnav->GetString(_descriptionNDB, _index[26], err);
  return err; 
}

int ItemBuildObj::GetType() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_type.empty()) _type.clear();
  if (_index[1] == 0) _type.clear();
  else fnav->GetString(_type, _index[1], err);
  return err; 
}

int ItemBuildObj::GetPrimitiveCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_primitiveCode.empty()) _primitiveCode.clear();
  if (_index[2] == 0) _primitiveCode.clear();
  else fnav->GetString(_primitiveCode, _index[2], err);
  return err; 
}

int ItemBuildObj::GetRegex() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_regex.empty()) _regex.clear();
  if (_index[3] == 0) _regex.clear();
  else fnav->GetString(_regex, _index[3], err);
  return err; 
}

int ItemBuildObj::GetExamplesCase()
{
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesCase.empty()) {
    for (i = 0; i < _examplesCase.size(); i++)
      _examplesCase[i].clear();
    _examplesCase.clear();
  }
  if (!_examplesDetail.empty()) {
    for (i = 0; i < _examplesDetail.size(); i++)
      _examplesDetail[i].clear();
    _examplesDetail.clear();
  }
  if (_index[4] == 0) _examplesCase.clear();
  else fnav->GetStrings(_examplesCase, _index[4], err);
  return err;
}

int ItemBuildObj::GetExamplesDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (_index[5] == 0) _examplesDetail.clear();
  else fnav->GetStrings(_examplesDetail, _index[5], err);
  if (_examplesDetail.size() != _examplesCase.size()) return -2;
  else return err;
}

int ItemBuildObj::GetEnumeration() {
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_enumeration.empty()) {
    for (i = 0; i < _enumeration.size(); i++)
      _enumeration[i].clear();
    _enumeration.clear();
  }
  if (_index[6] == 0) _enumeration.clear();
  else fnav->GetStrings(_enumeration, _index[6], err);
  return err;
}

int ItemBuildObj::GetEnumerationDetail() {
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_enumerationDetail.empty()) {
    for (i = 0; i < _enumerationDetail.size(); i++)
      _enumerationDetail[i].clear();
    _enumerationDetail.clear();
  }
  if (_index[7] == 0) _enumerationDetail.clear();
  else fnav->GetStrings(_enumerationDetail, _index[7], err);
  if (_enumerationDetail.size() != _enumeration.size()) return -2;
  return err;
}


int ItemBuildObj::GetExamplesCaseNDB()
{
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesCaseNDB.empty()) {
    for (i = 0; i < _examplesCaseNDB.size(); i++)
      _examplesCaseNDB[i].clear();
    _examplesCaseNDB.clear();
  }
  if (!_examplesDetailNDB.empty()) {
    for (i = 0; i < _examplesDetailNDB.size(); i++)
      _examplesDetailNDB[i].clear();
    _examplesDetailNDB.clear();
  }
  if (_index[27] == 0) _examplesCaseNDB.clear();
  else fnav->GetStrings(_examplesCaseNDB, _index[27], err);
  return err;
}

int ItemBuildObj::GetExamplesDetailNDB() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (_index[28] == 0) _examplesDetailNDB.clear();
  else fnav->GetStrings(_examplesDetailNDB, _index[28], err);
  if (_examplesDetailNDB.size() != _examplesCaseNDB.size()) return -2;
  else return err;
}

int ItemBuildObj::GetEnumerationNDB() {
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_enumerationNDB.empty()) {
    for (i = 0; i < _enumerationNDB.size(); i++)
      _enumerationNDB[i].clear();
    _enumerationNDB.clear();
  }
  if (_index[29] == 0) _enumerationNDB.clear();
  else fnav->GetStrings(_enumerationNDB, _index[29], err);
  return err;
}

int ItemBuildObj::GetEnumerationDetailNDB() {
  int err;
  err = 0;
  uWord i;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_enumerationDetailNDB.empty()) {
    for (i = 0; i < _enumerationDetailNDB.size(); i++)
      _enumerationDetailNDB[i].clear();
    _enumerationDetailNDB.clear();
  }
  if (_index[30] == 0) _enumerationDetailNDB.clear();
  else fnav->GetStrings(_enumerationDetailNDB, _index[30], err);
  if (_enumerationDetailNDB.size() != _enumerationNDB.size()) return -2;
  return err;
}


int ItemBuildObj::GetMandatoryCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_mandatoryCode.empty()) _mandatoryCode.clear();
  if (_index[8] == 0) _mandatoryCode.clear();
  else fnav->GetString(_mandatoryCode, _index[8], err);
  return err; 
}


int ItemBuildObj::GetDefaultValue() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_defaultValue.empty()) _defaultValue.clear();
  if (_index[9] == 0) _defaultValue.clear();
  else fnav->GetString(_defaultValue, _index[9], err);
  return err; 
}

int ItemBuildObj::GetUnits() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_units.empty()) _units.clear();
  if (_index[10] == 0) _units.clear();
  else fnav->GetString(_units, _index[10], err);
  return err; 
}

int ItemBuildObj::GetRangeMin() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_rangeMin.empty()) {
    for (uWord i = 0; i < _rangeMin.size(); i++) 
      _rangeMin[i].clear();
    _rangeMin.clear();
  }
  if (_index[11] == 0) _rangeMin.clear();
  else fnav->GetStrings(_rangeMin, _index[11], err);
  return err; 
}

int ItemBuildObj::GetRangeMax() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_rangeMax.empty()) {
    for (uWord i = 0; i < _rangeMax.size(); i++) 
      _rangeMax[i].clear();
    _rangeMax.clear();
  }
  if (_index[12] == 0) _rangeMax.clear();
  else fnav->GetStrings(_rangeMax, _index[12], err);
  return err; 
}

int ItemBuildObj::GetAliases() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_aliases.empty()) {
    for (uWord i = 0; i < _aliases.size(); i++) 
      _aliases[i].clear();
    _aliases.clear();
  }
  if (_index[13] == 0) _aliases.clear();
  else fnav->GetStrings(_aliases, _index[13], err);
  return err; 
}

int ItemBuildObj::GetAliasesDictionary() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_aliasesDictionary.empty()) {
    for (uWord i = 0; i < _aliasesDictionary.size(); i++) 
      _aliasesDictionary[i].clear();
    _aliasesDictionary.clear();
  }
  if (_index[14] == 0) _aliasesDictionary.clear();
  else fnav->GetStrings(_aliasesDictionary, _index[14], err);
  return err; 
}

int ItemBuildObj::GetAliasesDictionaryVersion() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_aliasesDictionaryVersion.empty()) {
    for (uWord i = 0; i < _aliasesDictionaryVersion.size(); i++) 
      _aliasesDictionaryVersion[i].clear();
    _aliasesDictionaryVersion.clear();
  }
  if (_index[15] == 0) _aliasesDictionaryVersion.clear();
  else fnav->GetStrings(_aliasesDictionaryVersion, _index[15], err);
  return err; 
}

int ItemBuildObj::GetItemStructure() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemStructure.empty()) _itemStructure.clear();
  if (_index[16] == 0) _itemStructure.clear();
  else fnav->GetString(_itemStructure, _index[16], err);
  return err; 
}

int ItemBuildObj::GetItemStructureOrganization() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemStructureOrganization.empty()) _itemStructureOrganization.clear();
  if (_index[17] == 0) _itemStructureOrganization.clear();
  else fnav->GetString(_itemStructureOrganization, _index[17], err);
  return err; 
}

int ItemBuildObj::GetDependents() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_dependents.empty()) {
    for (uWord i = 0; i < _dependents.size(); i++) 
      _dependents[i].clear();
    _dependents.clear();
  }
  if (_index[18] == 0) _dependents.clear();
  else fnav->GetStrings(_dependents, _index[18], err);
  return err; 
}

int ItemBuildObj::GetRelated() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_related.empty()) {
    for (uWord i = 0; i < _related.size(); i++) 
      _related[i].clear();
    _related.clear();
  }
  if (_index[19] == 0) _related.clear();
  else fnav->GetStrings(_related, _index[19], err);
  return err; 
}

int ItemBuildObj::GetRelatedFunctionCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_relatedFunctionCode.empty()) {
    for (uWord i = 0; i < _relatedFunctionCode.size(); i++) 
      _relatedFunctionCode[i].clear();
    _relatedFunctionCode.clear();
  }
  if (_index[20] == 0) _relatedFunctionCode.clear();
  else fnav->GetStrings(_relatedFunctionCode, _index[20], err);
  return err; 
}

int ItemBuildObj::GetSubcategories() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_subcategories.empty()) {
    for (uWord i = 0; i < _subcategories.size(); i++) 
      _subcategories[i].clear();
    _subcategories.clear();
  }
  if (_index[21] == 0) _subcategories.clear();
  else fnav->GetStrings(_subcategories, _index[21], err);
  return err; 
}

int ItemBuildObj::GetLinkedChildren() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_linkedChildren.empty()) {
    for (uWord i = 0; i < _linkedChildren.size(); i++) 
      _linkedChildren[i].clear();
    _linkedChildren.clear();
  }
  if (_index[22] == 0) _linkedChildren.clear();
  else fnav->GetStrings(_linkedChildren, _index[22], err);
  return err; 
}

int ItemBuildObj::GetLinkedParents() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_linkedParents.empty()) {
    for (uWord i = 0; i < _linkedParents.size(); i++) 
      _linkedParents[i].clear();
    _linkedParents.clear();
  }
  if (_index[23] == 0) _linkedParents.clear();
  else fnav->GetStrings(_linkedParents, _index[23], err);
  return err; 
}

int ItemBuildObj::GetMethods() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methods.empty()) {
    for (uWord i = 0; i < _methods.size(); i++) 
      _methods[i].clear();
    _methods.clear();
  }
  if (_index[24] == 0) _methods.clear();
  else fnav->GetStrings(_methods, _index[24], err);
  return err; 
}

int ItemBuildObj::GetTypeConditions() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_typeConditions.empty()) {
    for (uWord i = 0; i < _typeConditions.size(); i++) 
      _typeConditions[i].clear();
    _typeConditions.clear();
  }
  if (_index[25] == 0) _typeConditions.clear();
  else fnav->GetStrings(_typeConditions, _index[25], err);
  return err; 
}



SubcategoryBuildObj::SubcategoryBuildObj() {
/*
------------------------------------------------------------------------------
  constructor that initializes the object
------------------------------------------------------------------------------
*/
  Clear();
}

SubcategoryBuildObj::SubcategoryBuildObj(const string& dataBlockName, const string& subcategoryId) {
/*
------------------------------------------------------------------------------
  constructor that initializes the object and sets the keys
------------------------------------------------------------------------------
*/
  Clear();
  SetId(dataBlockName, subcategoryId);
}  

void SubcategoryBuildObj::SetId(const string& dataBlockName, const string& subcategoryId)
/*
------------------------------------------------------------------------------
  SetId() sets the keys of the object
------------------------------------------------------------------------------
*/
{
  _id = subcategoryId;  
  _datablock = dataBlockName;
}

int SubcategoryBuildObj::GetObject(uWord which) {
/*
-------------------------------------------------------------------------------
  GetObject() will call all of the private Get methods so that the entire
  object can be retrieved from persistent storage.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index) free(_index);
  uWord num;
  _index = fnav->GetWords(which, num, err); 
  GetDescription();
  GetExamplesCase();
  GetExamplesDetail();
  GetMethods();

  if (err == NO_ERROR) 
    return NO_ERROR; 
  else return err;
}

/*
  The following Get methods will retrieve the data members from persistent
  storage by calling on the FileNavigator class.
*/

int SubcategoryBuildObj::GetDescription() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_description.empty()) _description.clear();
  if (_index[0] == 0) _description.clear();
  else fnav->GetString(_description, _index[0], err);
  return err; 
}

int SubcategoryBuildObj::GetExamplesCase() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesCase.empty()) {
    for (uWord i = 0; i < _examplesCase.size(); i++) 
      _examplesCase[i].clear();
    _examplesCase.clear();
  }
  if (_index[1] == 0) _examplesCase.clear();
  else fnav->GetStrings(_examplesCase, _index[1], err);
  return err; 
}

int SubcategoryBuildObj::GetExamplesDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_examplesDetail.empty()) {
    for (uWord i = 0; i < _examplesDetail.size(); i++) 
      _examplesDetail[i].clear();
    _examplesDetail.clear();
  }
  if (_index[2] == 0) _examplesDetail.clear();
  else fnav->GetStrings(_examplesDetail, _index[2], err);
  return err; 
}

int SubcategoryBuildObj::GetMethods() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methods.empty()) {
    for (uWord i = 0; i < _methods.size(); i++) 
      _methods[i].clear();
    _methods.clear();
  }
  if (_index[3] == 0) _methods.clear();
  else fnav->GetStrings(_methods, _index[3], err);
  return err; 
}



DictionaryBuildObj::DictionaryBuildObj() {
/*
------------------------------------------------------------------------------
  constructor that initializes the object
------------------------------------------------------------------------------
*/
  Clear();
}

DictionaryBuildObj::DictionaryBuildObj(const string& dName) {
/*
------------------------------------------------------------------------------
  constructor that initializes th object and sets the key
------------------------------------------------------------------------------
*/
  Clear();
  SetName(dName);
}  

void DictionaryBuildObj::SetName(const string& dname) {
/*
------------------------------------------------------------------------------
  SetName() sets the key of the object
------------------------------------------------------------------------------
*/
  _name = dname;
}

int DictionaryBuildObj::GetObject(uWord which) {
/*
-------------------------------------------------------------------------------
  GetObject() will call all of the private Get methods so that the entire
  object can be retrieved from persistent storage.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index) free(_index);
  uWord num;
  _index = fnav->GetWords(which, num, err); 
  GetTitle();
  GetVersion();
  GetHistoryVersion();
  GetHistoryUpdate();
  GetHistoryRevision();
  GetCategories();
  GetSubcategories();
  GetItems();
  GetMethods();
  GetMethodsId();
  GetMethodsList();
  GetMethodsListDetail();
  GetMethodsListInline();
  GetMethodsListCode();
  GetMethodsListLanguage();
  GetCategoryGroupList();
  GetCategoryGroupListParents();
  GetCategoryGroupListDescription();
  GetItemStructureListCode();
  GetItemStructureListIndex();
  GetItemStructureListDimension();
  GetItemTypeListCode();
  GetItemTypeListPrimitiveCode();
  GetItemTypeListConstruct();
  GetItemTypeListDetail();
  GetItemUnitsListCode();
  GetItemUnitsListDetail();
  GetItemUnitsConversionOperator();
  GetItemUnitsConversionFactor();
  GetItemUnitsConversionFromCode();
  GetItemUnitsConversionToCode();

  BuildItemList();
  BuildSubcategoryList();
  BuildCategoryList();

  if (err == NO_ERROR) 
    return NO_ERROR; 
  else return err;
}

/*
  The following Get methods will retrieve the data members from persistent
  storage by calling on the FileNavigator class.
*/

int DictionaryBuildObj::GetTitle() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_title.empty()) _title.clear();
  if (_index[0] == 0) _title.clear();
  else fnav->GetString(_title, _index[0], err);
  return err; 
}

int DictionaryBuildObj::GetVersion() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_version.empty()) _version.clear();
  if (_index[1] == 0) _version.clear();
  else fnav->GetString(_version, _index[1], err);
  return err; 
}

int DictionaryBuildObj::GetDescription() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_description.empty()) _description.clear();
  if (_index[2] == 0) _description.clear();
  else fnav->GetString(_description, _index[2], err);
  return err; 
}

int DictionaryBuildObj::GetHistoryVersion() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_historyVersion.empty()) {
    for (uWord i = 0; i < _historyVersion.size(); i++) 
      _historyVersion[i].clear();
    _historyVersion.clear();
  }
  if (_index[3] == 0) _historyVersion.clear();
  else fnav->GetStrings(_historyVersion, _index[3], err);
  return err; 
}

int DictionaryBuildObj::GetHistoryUpdate() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_historyUpdate.empty()) {
    for (uWord i = 0; i < _historyUpdate.size(); i++) 
      _historyUpdate[i].clear();
    _historyUpdate.clear();
  }
  if (_index[4] == 0) _historyUpdate.clear();
  else fnav->GetStrings(_historyUpdate, _index[4], err);
  return err; 
}

int DictionaryBuildObj::GetHistoryRevision() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_historyRevision.empty()) {
    for (uWord i = 0; i < _historyRevision.size(); i++) 
      _historyRevision[i].clear();
    _historyRevision.clear();
  }
  if (_index[5] == 0) _historyRevision.clear();
  else fnav->GetStrings(_historyRevision, _index[5], err);
  return err; 
}

int DictionaryBuildObj::GetCategories() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categories.empty()) {
    for (uWord i = 0; i < _categories.size(); i++) 
      _categories[i].clear();
    _categories.clear();
  }
  if (_index[6] == 0) _categories.clear();
  else fnav->GetStrings(_categories, _index[6], err);
  return err; 
}

int DictionaryBuildObj::GetSubcategories() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_subcategories.empty()) {
    for (uWord i = 0; i < _subcategories.size(); i++) 
      _subcategories[i].clear();
    _subcategories.clear();
  }
  if (_index[7] == 0) _subcategories.clear();
  else fnav->GetStrings(_subcategories, _index[7], err);
  return err; 
}

int DictionaryBuildObj::GetItems() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_items.empty()) {
    for (uWord i = 0; i < _items.size(); i++) 
      _items[i].clear();
    _items.clear();
  }
  if (_index[8] == 0) _items.clear();
  else fnav->GetStrings(_items, _index[8], err);
  return err; 
}

int DictionaryBuildObj::GetMethods() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methods.empty()) {
    for (uWord i = 0; i < _methods.size(); i++) 
      _methods[i].clear();
    _methods.clear();
  }
  if (_index[9] == 0) _methods.clear();
  else fnav->GetStrings(_methods, _index[9], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsId() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsId.empty()) {
    for (uWord i = 0; i < _methodsId.size(); i++) 
      _methodsId[i].clear();
    _methodsId.clear();
  }
  if (_index[10] == 0) _methodsId.clear();
  else fnav->GetStrings(_methodsId, _index[10], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsList() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsList.empty()) {
    for (uWord i = 0; i < _methodsList.size(); i++) 
      _methodsList[i].clear();
    _methodsList.clear();
  }
  if (_index[11] == 0) _methodsList.clear();
  else fnav->GetStrings(_methodsList, _index[11], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsListDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsListDetail.empty()) {
    for (uWord i = 0; i < _methodsListDetail.size(); i++) 
      _methodsListDetail[i].clear();
    _methodsListDetail.clear();
  }
  if (_index[12] == 0) _methodsListDetail.clear();
  else fnav->GetStrings(_methodsListDetail, _index[12], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsListInline() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsListInline.empty()) {
    for (uWord i = 0; i < _methodsListInline.size(); i++) 
      _methodsListInline[i].clear();
    _methodsListInline.clear();
  }
  if (_index[13] == 0) _methodsListInline.clear();
  else fnav->GetStrings(_methodsListInline, _index[13], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsListCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsListCode.empty()) {
    for (uWord i = 0; i < _methodsListCode.size(); i++) 
      _methodsListCode[i].clear();
    _methodsListCode.clear();
  }
  if (_index[14] == 0) _methodsListCode.clear();
  else fnav->GetStrings(_methodsListCode, _index[14], err);
  return err; 
}

int DictionaryBuildObj::GetMethodsListLanguage() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_methodsListLanguage.empty()) {
    for (uWord i = 0; i < _methodsListLanguage.size(); i++) 
      _methodsListLanguage[i].clear();
    _methodsListLanguage.clear();
  }
  if (_index[15] == 0) _methodsListLanguage.clear();
  else fnav->GetStrings(_methodsListLanguage, _index[15], err);
  return err; 
}

int DictionaryBuildObj::GetCategoryGroupList() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categoryGroupList.empty()) {
    for (uWord i = 0; i < _categoryGroupList.size(); i++) 
      _categoryGroupList[i].clear();
    _categoryGroupList.clear();
  }
  if (_index[16] == 0) _categoryGroupList.clear();
  else fnav->GetStrings(_categoryGroupList, _index[16], err);
  return err; 
}

int DictionaryBuildObj::GetCategoryGroupListParents() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categoryGroupListParents.empty()) {
    for (uWord i = 0; i < _categoryGroupListParents.size(); i++) 
      _categoryGroupListParents[i].clear();
    _categoryGroupListParents.clear();
  }
  if (_index[17] == 0) _categoryGroupListParents.clear();
  else fnav->GetStrings(_categoryGroupListParents, _index[17], err);
  return err; 
}

int DictionaryBuildObj::GetCategoryGroupListDescription() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_categoryGroupListDescription.empty()) {
    for (uWord i = 0; i < _categoryGroupListDescription.size(); i++) 
      _categoryGroupListDescription[i].clear();
    _categoryGroupListDescription.clear();
  }
  if (_index[18] == 0) _categoryGroupListDescription.clear();
  else fnav->GetStrings(_categoryGroupListDescription, _index[18], err);
  return err; 
}

int DictionaryBuildObj::GetItemStructureListCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemStructureListCode.empty()) {
    for (uWord i = 0; i < _itemStructureListCode.size(); i++) 
      _itemStructureListCode[i].clear();
    _itemStructureListCode.clear();
  }
  if (_index[19] == 0) _itemStructureListCode.clear();
  else fnav->GetStrings(_itemStructureListCode, _index[19], err);
  return err; 
}

int DictionaryBuildObj::GetItemStructureListIndex() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemStructureListIndex.empty()) {
    for (uWord i = 0; i < _itemStructureListIndex.size(); i++) 
      _itemStructureListIndex[i].clear();
    _itemStructureListIndex.clear();
  }
  if (_index[20] == 0) _itemStructureListIndex.clear();
  else fnav->GetStrings(_itemStructureListIndex, _index[20], err);
  return err; 
}

int DictionaryBuildObj::GetItemStructureListDimension() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemStructureListDimension.empty()) {
    for (uWord i = 0; i < _itemStructureListDimension.size(); i++) 
      _itemStructureListDimension[i].clear();
    _itemStructureListDimension.clear();
  }
  if (_index[21] == 0) _itemStructureListDimension.clear();
  else fnav->GetStrings(_itemStructureListDimension, _index[21], err);
  return err; 
}

int DictionaryBuildObj::GetItemTypeListCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemTypeListCode.empty()) {
    for (uWord i = 0; i < _itemTypeListCode.size(); i++) 
      _itemTypeListCode[i].clear();
    _itemTypeListCode.clear();
  }
  if (_index[22] == 0) _itemTypeListCode.clear();
  else fnav->GetStrings(_itemTypeListCode, _index[22], err);
  return err; 
}

int DictionaryBuildObj::GetItemTypeListPrimitiveCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemTypeListPrimitiveCode.empty()) {
    for (uWord i = 0; i < _itemTypeListPrimitiveCode.size(); i++) 
      _itemTypeListPrimitiveCode[i].clear();
    _itemTypeListPrimitiveCode.clear();
  }
  if (_index[23] == 0) _itemTypeListPrimitiveCode.clear();
  else fnav->GetStrings(_itemTypeListPrimitiveCode, _index[23], err);
  return err; 
}

int DictionaryBuildObj::GetItemTypeListConstruct() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemTypeListConstruct.empty()) {
    for (uWord i = 0; i < _itemTypeListConstruct.size(); i++) 
      _itemTypeListConstruct[i].clear();
    _itemTypeListConstruct.clear();
  }
  if (_index[24] == 0) _itemTypeListConstruct.clear();
  else fnav->GetStrings(_itemTypeListConstruct, _index[24], err);
  return err; 
}

int DictionaryBuildObj::GetItemTypeListDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemTypeListDetail.empty()) {
    for (uWord i = 0; i < _itemTypeListDetail.size(); i++) 
      _itemTypeListDetail[i].clear();
    _itemTypeListDetail.clear();
  }
  if (_index[25] == 0) _itemTypeListDetail.clear();
  else fnav->GetStrings(_itemTypeListDetail, _index[25], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsListCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsListCode.empty()) {
    for (uWord i = 0; i < _itemUnitsListCode.size(); i++) 
      _itemUnitsListCode[i].clear();
    _itemUnitsListCode.clear();
  }
  if (_index[26] == 0) _itemUnitsListCode.clear();
  else fnav->GetStrings(_itemUnitsListCode, _index[26], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsListDetail() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsListDetail.empty()) {
    for (uWord i = 0; i < _itemUnitsListDetail.size(); i++) 
      _itemUnitsListDetail[i].clear();
    _itemUnitsListDetail.clear();
  }
  if (_index[27] == 0) _itemUnitsListDetail.clear();
  else fnav->GetStrings(_itemUnitsListDetail, _index[27], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsConversionOperator() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsConversionOperator.empty()) {
    for (uWord i = 0; i < _itemUnitsConversionOperator.size(); i++) 
      _itemUnitsConversionOperator[i].clear();
    _itemUnitsConversionOperator.clear();
  }
  if (_index[28] == 0) _itemUnitsConversionOperator.clear();
  else fnav->GetStrings(_itemUnitsConversionOperator, _index[28], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsConversionFactor() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsConversionFactor.empty()) {
    for (uWord i = 0; i < _itemUnitsConversionFactor.size(); i++) 
      _itemUnitsConversionFactor[i].clear();
    _itemUnitsConversionFactor.clear();
  }
  if (_index[29] == 0) _itemUnitsConversionFactor.clear();
  else fnav->GetStrings(_itemUnitsConversionFactor, _index[29], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsConversionFromCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsConversionFromCode.empty()) {
    for (uWord i = 0; i < _itemUnitsConversionFromCode.size(); i++) 
      _itemUnitsConversionFromCode[i].clear();
    _itemUnitsConversionFromCode.clear();
  }
  if (_index[30] == 0) _itemUnitsConversionFromCode.clear();
  else fnav->GetStrings(_itemUnitsConversionFromCode, _index[30], err);
  return err; 
}

int DictionaryBuildObj::GetItemUnitsConversionToCode() {
  int err;
  err = 0;
  if (!_index) return INDEX_NOT_CONSTRUCTED;
  if (!_itemUnitsConversionToCode.empty()) {
    for (uWord i = 0; i < _itemUnitsConversionToCode.size(); i++) 
      _itemUnitsConversionToCode[i].clear();
    _itemUnitsConversionToCode.clear();
  }
  if (_index[31] == 0) _itemUnitsConversionToCode.clear();
  else fnav->GetStrings(_itemUnitsConversionToCode, _index[31], err);
  return err; 
}

ItemBuildObj * DictionaryBuildObj::GetItem(const string& itemName) {
/*
------------------------------------------------------------------------------
  GetItem() searches through the object's list of items and returns the
  ItemBuildObject if it is in memory. Otherwise, it constructs the object,
  adds it to the list of in-memory objects, and returns the newly constructed
  object.
------------------------------------------------------------------------------
*/
  // VLAD BINSEARCH
  // Word ind;
  Word ind = binSearch(itemName, _itemsList.keys, sizeof(string), sCompare);
  if (ind < 0 || ind > (int) _itemsList.keys.size()) return NULL;

  if (_itemsList.memIds[ind] != 0)
    return _itemObjs[_itemsList.memIds[ind]];
  _itemsList.memIds[ind] = _lastItemInMem;  
  _itemObjs[_lastItemInMem] = new ItemBuildObj;
  (_itemObjs[_lastItemInMem])->SetName(_name, itemName);
  (_itemObjs[_lastItemInMem])->GetObject(_itemsList.objectIds[ind]);

  return _itemObjs[_lastItemInMem++];   
}

CategoryBuildObj * DictionaryBuildObj::GetCategory(const string& categoryName){
/*
------------------------------------------------------------------------------
  GetCategory() searches through the object's list of categories and returns
  the CategoryBuildObj if it is in memory. Otherwise, it constructs the
  object, adds it to the list of in-memory objects, and returns the newly
  constructed object.
------------------------------------------------------------------------------
*/
  // VLAD BINSEARCH
  // Word ind;

  Word ind = binSearch(categoryName, _categoriesList.keys, sizeof(string), sCompare);
  if (ind < 0 || ind > (int) _categoriesList.keys.size()) return NULL;
  if (_categoriesList.memIds[ind] != 0)
    return _categoryObjs[_categoriesList.memIds[ind]];
  _categoriesList.memIds[ind] = _lastCategoryInMem;  
  _categoryObjs[_lastCategoryInMem] = new CategoryBuildObj;
  (_categoryObjs[_lastCategoryInMem])->SetId(_name, categoryName);
  (_categoryObjs[_lastCategoryInMem])->GetObject(_categoriesList.objectIds[ind]);

  return _categoryObjs[_lastCategoryInMem++];   
}

SubcategoryBuildObj * DictionaryBuildObj::GetSubcategory(const string& subcategoryName){
/*
------------------------------------------------------------------------------
  GetSub_category() searches through the object's list of subcategories and
  returns the SubcategoryBuildObj if it is in memory. Otherwise, it constructs
  the object, adds it to the list of in-memory objects, and returns the newly
  constructed object.
------------------------------------------------------------------------------
*/
  // VLAD BINSEARCH
  // Word ind;
  Word ind = binSearch(subcategoryName, _subcategoriesList.keys, sizeof(string), sCompare);
  if (ind < 0 || ind > (int) _subcategoriesList.keys.size()) return NULL;
  if (_subcategoriesList.memIds[ind] != 0)
    return _subcategoryObjs[_subcategoriesList.memIds[ind]];
  _subcategoriesList.memIds[ind] = _lastSubcategoryInMem;  
  _subcategoryObjs[_lastSubcategoryInMem] = new SubcategoryBuildObj;
  (_subcategoryObjs[_lastSubcategoryInMem])->SetId(_name, subcategoryName);
  (_subcategoryObjs[_lastSubcategoryInMem])->GetObject(_subcategoriesList.objectIds[ind]);

  return _subcategoryObjs[_lastSubcategoryInMem++];   
}

void DictionaryBuildObj::Reset() {
/*
-------------------------------------------------------------------------------
  Reset() frees all owned memory. Overr_ides DictionaryObjBase::Reset()
-------------------------------------------------------------------------------
*/
  Word i;
  DictionaryObjBase::Reset();
  if (!_itemsList.objectIds.empty()) _itemsList.objectIds.clear();
  if (_itemsList.memIds) free(_itemsList.memIds);
  if (_itemObjs) {
    for (i = 0; i < (int) _lastItemInMem; i++) {
      if (_itemObjs[i]) free(_itemObjs[i]);
    }
    free(_itemObjs);
  }
  if (_subcategoryObjs) {
    for (i = 0; i < (int) _lastSubcategoryInMem; i++) {
      if (_subcategoryObjs[i]) free(_subcategoryObjs[i]);
    }
    free(_subcategoryObjs);
  }    
  if (_categoryObjs) {
    for (i = 0; i < (int) _lastCategoryInMem; i++) {
      if (_categoryObjs[i]) free(_categoryObjs[i]);
    }
    free(_categoryObjs);
  }    

  Clear();
}

void DictionaryBuildObj::Clear() {
/*
-------------------------------------------------------------------------------
  Clear() initializes the object. Overr_ides DictionaryObjBase::Clear()
-------------------------------------------------------------------------------
*/
  DictionaryObjBase::Clear();
  _itemsList.keys.clear();
  _itemsList.objectIds.clear();
  _itemsList.memIds = NULL;
  _subcategoriesList.keys.clear();
  _subcategoriesList.objectIds.clear();
  _subcategoriesList.memIds = NULL;
  _categoriesList.keys.clear();
  _categoriesList.objectIds.clear();
  _categoriesList.memIds = NULL;
  _lastItemInMem = 1;
  _lastSubcategoryInMem = 1;
  _lastCategoryInMem = 1;
  _itemObjs = NULL;
  _subcategoryObjs = NULL;
  _categoryObjs = NULL;
}

int DictionaryBuildObj::BuildItemList() {
/*
-------------------------------------------------------------------------------
  BuildItemList() retrieves from disk the information needed to load in the
  itemObjs from disk into memory.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index[32] == 0) return BUILD_LIST_ERROR;
  fnav->GetUWords(_itemsList.objectIds, _index[32], err);

  _itemsList.keys = _items;

  _itemsList.memIds = (uWord *) calloc(_itemsList.keys.size(), WORDSIZE);
  _itemObjs = (ItemBuildObj **) calloc(_itemsList.keys.size() + 1, sizeof(ItemBuildObj *));
  return NO_ERROR;
}

int DictionaryBuildObj::BuildSubcategoryList() {
/*
-------------------------------------------------------------------------------
  BuildSubcategoryList() retrieves from disk the information needed to load in
  the SubcategoryObjs from disk into memory.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index[33] == 0) return BUILD_LIST_ERROR;
  fnav->GetUWords(_subcategoriesList.objectIds, _index[33], err);

  _subcategoriesList.keys = _subcategories;

  _subcategoriesList.memIds = (uWord *) calloc(_subcategoriesList.keys.size(), WORDSIZE);
  _subcategoryObjs = (SubcategoryBuildObj **) calloc(_subcategoriesList.keys.size() + 1, sizeof(SubcategoryBuildObj *));
  return NO_ERROR;
}

int DictionaryBuildObj::BuildCategoryList() {
/*
-------------------------------------------------------------------------------
  BuildCategoryList() retrieves from disk the information needed to load in
  the CategoryObjs from disk into memory.
-------------------------------------------------------------------------------
*/
  int err;
  err = 0;
  if (_index[34] == 0) return BUILD_LIST_ERROR;
  fnav->GetUWords(_categoriesList.objectIds, _index[34], err);

  _categoriesList.keys = _categories;

  _categoriesList.memIds = (uWord *) calloc(_categoriesList.keys.size(), WORDSIZE);
  _categoryObjs = (CategoryBuildObj **) calloc(_categoriesList.keys.size() + 1, sizeof(CategoryBuildObj *));
  return NO_ERROR;
}

int DictionaryBuildObj::GetAllItems() {
/*
-------------------------------------------------------------------------------
  GetAllItems() is a method that gets all _items in this _datablock/dictionary
  and prints them out
-------------------------------------------------------------------------------
*/
  uWord i = 0;
  ItemBuildObj * ibo;
  for (i = 0; i < _items.size(); i++) {
    ibo = GetItem(_items[i].c_str());
    if (ibo) {
      ibo->PrintObject();
      ibo->Reset();
    }  else {
      printf("Failed to load item %s\n",_items[i].c_str());
    }
  }
  return NO_ERROR;
}

int DictionaryBuildObj::GetAllSubcategories() {
/*
-------------------------------------------------------------------------------
  GetAllSubcategories() is a method that gets all _subcategories in this
  datablock/dictionary and prints them out
-------------------------------------------------------------------------------
*/
  uWord i = 0;
  SubcategoryBuildObj * sbo;
  for (i = 0; i < _subcategories.size(); i++) {
    sbo = GetSubcategory(_subcategories[i].c_str());
    if (sbo) {
      sbo->PrintObject();
      sbo->Reset();
    }  else {
      printf("Failed to load subcategory %s\n",_subcategories[i].c_str());
    }
  }
  return NO_ERROR;
}

int DictionaryBuildObj::GetAllCategories() {
/*
-------------------------------------------------------------------------------
  GetAllCategories() is a method that gets all categories in this 
  datablock/dictionary and prints them out
-------------------------------------------------------------------------------
*/
  uWord i = 0;
  CategoryBuildObj * cbo;
  for (i = 0; i < _categories.size(); i++) {
    cbo = GetCategory(_categories[i].c_str());
    if (cbo) {
      cbo->PrintObject();
      cbo->Reset();
    } else {
      printf("Failed to load category %s\n",_categories[i].c_str());
    }
  }
  return NO_ERROR;
}
