#include <string>

#include "CifString.h"
#include "ISTable.h"
#include "misc_util.h"


static const string PDBML_PARSER_LOG_FILE("PdbMlParser.log");


static string RemoveLostCharacters(const string& inString);


string toString(const XMLCh* toConvert)
{

    StrX strX(toConvert);
    return(string(strX.localForm()));

}


void Error(string& err)
{

    ofstream out(PDBML_PARSER_LOG_FILE.c_str(), ios::app);
    out << err << endl;
    out.close();

}


void DeleteFile(const string& fileName)
{

    unlink(fileName.c_str());

}


void ResurrectOrigItemNames(CifFile& cifFile, CifBuilder& cifBuilder)
{

    vector<string> blockNames;
    vector<string> origComboItems;
    string origItemName;
    string nonResItemName;
    string categoryName;
    ISTable* tableP = NULL;
    int ret;


    cifFile.GetBlockNames(blockNames);

    for (unsigned int blockI = 0; blockI < blockNames.size(); blockI++)
    {
        // Get the list of all original combo items of all categories
        cifBuilder.GetDictionaryItems(origComboItems);

        for (unsigned int itemI = 0; itemI < origComboItems.size(); itemI++)
        {
            // Get original item name from combo item name
            ret = CifString::GetItemFromCifItem(origItemName,
              origComboItems[itemI]);

            if (ret != 1)
            {
                    // VLAD ERROR HANDLING
                    // Report error.
            }

            // Remove lost characters and search for non-resurrected item
            // in the converted CIF file.
            nonResItemName = RemoveLostCharacters(origItemName);

            if (origItemName.size() == nonResItemName.size())
            {
                // No lost characters found.
                continue;
            }

            // Rename the item in CIF file.

            // Get category name from combo item name
            ret = CifString::GetCategoryFromCifItem(categoryName,
              origComboItems[itemI]);

            if (ret != 1)
            {
                // VLAD ERROR HANDLING
                // Report error.
            }

            tableP = cifFile.GetTablePtr(blockNames[blockI],
              categoryName);

            if (tableP == NULL)
            {
                continue;
            }

            tableP->RenameColumn(nonResItemName, origItemName);
        } // for (each original item)

        origComboItems.clear();

    } // for (each block)

    blockNames.clear();

} /* End of ResurrectOrigItemNames() */


void CorrectEmptyValuesOfMandatoryItems(CifFile& cifFile,
  CifBuilder& cifBuilder)
{

    vector<string> blockNames;
    vector<string> tableNames;
    ISTable* tableP = NULL;
    vector<string> comboMandItems;
    int ret;
    string mandItemName;
    int colIndex;
    string cell;


    cifFile.GetBlockNames(blockNames);

    for (unsigned int blockI = 0; blockI < blockNames.size(); blockI++)
    {
        cifFile.GetTableNames(tableNames, blockNames[blockI]);

        for (unsigned int tableI = 0; tableI < tableNames.size(); tableI++)
        {
            tableP = cifFile.GetTablePtr(blockNames[blockI],
              tableNames[tableI]);

            // Get all the mandatory items for this category
            cifBuilder.GetCategoryKeys(comboMandItems, tableP->GetName());

            for (unsigned int itemI = 0; itemI < comboMandItems.size(); itemI++)
            {
                // Get original item name from combo item name
                ret = CifString::GetItemFromCifItem(mandItemName,
                  comboMandItems[itemI]);

                if (ret != 1)
                {
                        // VLAD ERROR HANDLING
                        // Report error.
                }

                colIndex = tableP->GetColumnIndex(mandItemName);
                if (colIndex == COLUMN_NAME_NOT_FOUND)
                {
                    // VLAD - ERROR HANDLING
                    continue;
                }

                for (unsigned int rowI = 0; rowI < tableP->GetNumRows();
                  rowI++)
                {
                    tableP->GetCell(cell, colIndex, rowI);
                    if (cell.empty())
                    {
                        tableP->UpdateCell(".", colIndex, rowI);
                    }
                } // For all values of mandatory items
            } // For all mandatory items

            comboMandItems.clear();
        } // For all tables

        tableNames.clear();
    } // For all blocks

    blockNames.clear();

} /* End of CorrectEmptyValuesOfMandatoryItems() */


string RemoveLostCharacters(const string& inString)
{

    if (inString.empty())
    {
        string errString = "ERROR: Detected empty string where it should" \
          " not have been detected!\n";
        Error(errString);
        return(inString);
    }

    string outString = inString;

    string tmpString(inString);

    string::iterator iter = remove_if(tmpString.begin(), tmpString.end(),
      IsLostChar());

    if (iter == tmpString.end())
    {
        return(outString);
    }

    // Remove all the lost characters from the string.
    outString.assign(tmpString.begin(), iter);

    return(outString);

} /* End of RemoveLostCharacters() */
