#include <string>

#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <sax2/XMLReaderFactory.hpp>

#include "CifFile.h"
#include "CifBuilder.h"

#include "misc_util.h"
#include "PdbMlParserHandler.h"


using namespace std;
XERCES_CPP_NAMESPACE_USE


static void ShowUsage(void);


int main(int argc, char* argv[])
{

    string xmlFileName;
    string dictObjFileName;
    string dictName;
    int retCode;


    // VLAD: PDB GENERALIZE: ARGUMENTS PARSING
    for (int i =  1; i < argc; i++)
    {
        if (argv[i][0] == '-' )
        {
            if (!strcmp(argv[i], "-xml"))
            {
                i++;
                xmlFileName = argv[i];
            }
            else if (!strcmp(argv[i], "-df"))
            {
                i++;
                dictObjFileName = argv[i];
            }
            else if (!strcmp(argv[i], "-dict"))
            {
                i++;
                dictName = argv[i];
            }
        }
    }

    if (xmlFileName.empty() || dictObjFileName.empty() || dictName.empty())
    {
        ShowUsage();
        exit(1);
    }

    string cifFileName = xmlFileName + ".cif";

    CifBuilder dictBuilder(dictObjFileName);

    dictBuilder.BuildDictionaryList();

    retCode = dictBuilder.SetContext(dictName);
    if (retCode != 0)
    {
        cerr << "Could not find dictionary " << dictName <<
          " in dictionary object file " << dictObjFileName << endl;
        exit(1);
    }

    try
    {
        XMLPlatformUtils::Initialize();
    }
    catch (const XMLException& toCatch)
    {
        string err("Error in PdbMlParser [main() method]: Error during"\
          " SAX2 initialization! :\n");
        err += toString(toCatch.getMessage());
        err += ".\n\n";

        Error(err);

        return(-1);
    }

    SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();

    int errorCount = 0;

    CifFile cifFile(VIRTUAL_MODE, "virtual", true);

    try
    {
        PdbMlParserHandler tHandler(cifFile);

        parser->setContentHandler(&tHandler);
        parser->setErrorHandler(&tHandler);

        cout << xmlFileName << endl;
        parser->parse(xmlFileName.c_str());
        cout << xmlFileName << endl;

        errorCount = parser->getErrorCount();
    }
    catch (const XMLException& toCatch)
    {
        string err("Error in PdbMlParser [main() method]: A SAX2 error"\
          " occurred:\n");
        err += toString(toCatch.getMessage());
        err += ".\n\n";

        Error(err);

        XMLPlatformUtils::Terminate();
        return(-1);
    }

    delete parser;
    XMLPlatformUtils::Terminate();

    // Resurrect original item names
    ResurrectOrigItemNames(cifFile, dictBuilder);

    CorrectEmptyValuesOfMandatoryItems(cifFile, dictBuilder);

    cifFile.Write(cifFileName);

    if (errorCount > 0)
        return(-1);
    else
        return(0);

} /* End of main() */


void ShowUsage(void)
{

    cout << "Usage: xml2mmcif -xml xml_file -df dict_object_file " \
      "-dict dict_name [-v]" << endl;

}
